//---------------------------------------------------------------------------
//
// MODULE NAME:     main.cpp
//
// COPYRIGHT:       Copyright (c) 2013, GladeSoft, Inc.
//                  All Rights Reserved.
//
//                  NOTICE: THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF
//                  GLADESOFT, INC., AND SHALL NOT BE REPRODUCED, COPIED
//                  IN WHOLE OR IN PART ADAPTED, MODIFIED, OR
//                  DISSEMINATED WITHOUT THE EXPRESS WRITTEN CONSENT
//                  OF GLADESOFT, INC. ANY REDISTRIBUTION OR
//                  PUBLICATION TO UNAUTHORIZED PERSONS IS STRICTLY
//                  PROHIBITED BY U.S. AND INTERNATIONAL COPYRIGHT LAW.
//
// DESCRIPTION:     Another example program using the Seminole embedded 
//                  web server.
//
// AUTHOR:          S. Conner
//
// CREATION DATE:   Jun  6, 2004
//
//---------------------------------------------------------------------------
//
// HISTORY OF CHANGE  
//
// -----+--------+------------------------------------------+------+---------
// VERS | DATE   | DESCRIPTION OF CHANGE                    | INIT | PR#
// =====+========+==========================================+======+=========
//  001 |06/06/04| File creation.                           | SPC  |
// -----+--------+------------------------------------------+------+---------
//  002 |06/12/09| Const correctness.                       | RAH  |
// -----+--------+------------------------------------------+------+---------
//---------------------------------------------------------------------------

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "seminole.h"

#include "sitedata.h"

//**********************************************************************

class DemoFileHandler : public HttpdFileHandler
{
protected:
  bool TranslateUri(RequestState &state);
    
public:
  DemoFileHandler(HttpdFileSystem *p_filesys,
                  const char      *p_root,
                  const char      *p_prefix);
};

static const char  gServername[] = "localhost";
static const int   gServerport   = 8080;
static Httpd      *gpWebServer;

//**********************************************************************

//---------------------------------------------------------------------------
//
// NAME:           DemoFileHandler::DemoFileHandler
//
// DESCRIPTION:    Construct the demo file handler.
//
// AUTHOR:         S. Conner
// CREATED:        Jun  6, 2006
// LAST MODIFIED:  N/A           BY:  N/A
//
//--------------------------------------------------------------------------
DemoFileHandler::DemoFileHandler(HttpdFileSystem *p_filesys,
                                 const char      *p_root,
                                 const char      *p_prefix)
 : HttpdFileHandler(p_filesys, p_root, p_prefix)
{}

//---------------------------------------------------------------------------
//
// NAME:           DemoFileHandler::TranslateUri
//
// DESCRIPTION:    Perform the URI translation phase.
//
// RETURNS:        Success indicator.
//
// AUTHOR:         S. Conner
// CREATED:        Jun  6, 2006
// LAST MODIFIED:  N/A           BY:  N/A
//
//--------------------------------------------------------------------------
bool DemoFileHandler::TranslateUri(RequestState &state)
{
  // Initialize mpFilePath so that all failure cases are
  // properly handled.
  state.mpFilePath = NULL;

  // Decode the request path.
  state.mpDecodedUri = HttpdUtilities::UriDecode(state.mpReqPath);
  if (httpd_rarely(state.mpDecodedUri == NULL))
    goto fail;

  // Lowercase the filename. 
  for(char *p_walk = state.mpDecodedUri; *p_walk != '\0'; p_walk++)
    *p_walk = tolower(*p_walk);

  // Convert to a filesystem path.
  state.mpFilePath = HttpdUtilities::Normalize(state.mpDecodedUri,
                                               mpRootPath);
  if (httpd_rarely(state.mpFilePath == NULL))
    goto fail;

  return (true);

fail:
  state.mpRequest->Respond(HTTPD_RESP_SRV_ERROR);
  return (false);
}

//*************************************************************************

//---------------------------------------------------------------------------
//
// NAME:           failure
//
// DESCRIPTION:    Handle a fatal error.
//
// AUTHOR:         S. Conner
// CREATED:        Mar  6, 2004
// LAST MODIFIED:  N/A           BY:  N/A
//
//--------------------------------------------------------------------------
static void failure(const char *p_file, int line, const char *p_msg)
{
  assert(p_file != NULL);
  assert(line   >  0);
  assert(p_msg  != NULL);
  
  fprintf(stderr, "%s:%d Call to %s failed\n", p_file, line, p_msg);
  exit(EXIT_FAILURE);
}

//---------------------------------------------------------------------------
//
// NAME:           Initialize_The_Web_Server
//
// DESCRIPTION:    Start and configure Seminole.
//
// AUTHOR:         M. Grosberg
// CREATED:        Jun  7, 2006
// LAST MODIFIED:  N/A           BY:  N/A
//
//--------------------------------------------------------------------------
static void Initialize_The_Web_Server()
{
  //-------------------------------------------------------
  // Initialize the environment.
  //-------------------------------------------------------
  
  if (Httpd::Init() != 0)
    failure(__FILE__, __LINE__, "Httpd::Init()");
  
  //--------------------------------------------------------
  // Create the webserver instance.  This just initializes
  // the object --- it doesn't start it up.
  //--------------------------------------------------------
    
  gpWebServer = new Httpd(gServername, gServerport);
  if (gpWebServer == NULL)
    failure(__FILE__, __LINE__, "Httpd::Httpd()");

  static HttpdMemoryDataSource builtin(SemWeb, sizeof(SemWeb));
  static HttpdRomFileSystem    romfs;
  if (romfs.Mount(&builtin) != 0)
    failure(__FILE__, __LINE__, "HttpdRomFileSystem::Mount()");

  static DemoFileHandler hand(&romfs, "/", "/");
  gpWebServer->Install(&hand);

  static HttpdFileHandler cs_hand(&romfs, "/", "/cs");
  gpWebServer->Install(&cs_hand);

  printf("Server started on host %s port %d\n", gServername, gServerport);
}

//---------------------------------------------------------------------------
//
// NAME:           main
//
// DESCRIPTION:    Entry point for the demo.
//
// AUTHOR:         S. Conner
// CREATED:        Mar  6, 2004
// LAST MODIFIED:  N/A           BY:  N/A
//
//--------------------------------------------------------------------------
int main(int argc, const char *const *argv)
{
  char buf[4];

  Initialize_The_Web_Server();
  gpWebServer->Start();
  
  printf("Press <ENTER> to stop.\n");
  fgets(buf, sizeof(buf), stdin);
  
#if HTTPD_INC_SHUTDOWN
  gpWebServer->Stop(HARD);
#endif

  return (EXIT_SUCCESS);
}
